package cn.com.qmth.examcloud.service.core.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.rabbitmq.client.AMQP.Basic.Return;

import cn.com.qmth.examcloud.service.core.entity.Specialty;
import cn.com.qmth.examcloud.service.core.repo.SpecialtyRepo;

@Service
public class SpecialtyService {

	@Autowired
	SpecialtyRepo specialtyRepo;
	
	/**
	 * 带分页查询专业
	 * @param specialty
	 * @param pageable
	 * @return
	 */
	public Page<Specialty> findAll(Specialty specialty,Pageable pageable){
		Specification<Specialty> specification = getSpecification(specialty);
		Page<Specialty> specialties = specialtyRepo.findAll(specification, pageable);
		return specialties;
	}
	
	public Specification<Specialty> getSpecification(Specialty specialty){
		Specification<Specialty> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if(!StringUtils.isEmpty(specialty.getOrgId())){
				predicates.add(cb.equal(root.get("orgId"),specialty.getOrgId()));
			}
			if(!StringUtils.isEmpty(specialty.getName())){
		    	predicates.add(cb.like(root.get("name"),"%"+specialty.getName()+"%"));
		    }
		    if(!StringUtils.isEmpty(specialty.getCode())){
		    	predicates.add(cb.like(root.get("code"),"%"+specialty.getCode()+"%"));
		    }
		    if(!StringUtils.isEmpty(specialty.getEnable())){
		    	predicates.add(cb.equal(root.get("enable"),specialty.getEnable()));
		    }
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}
	
	/**
	 * 查询专业不带分页
	 * @param specialty
	 * @return
	 */
	public List<Specialty> findAll(Specialty specialty){
		Specification<Specialty> specification = getSpecification(specialty);
		return specialtyRepo.findAll(specification);
	}
	
	/**
	 * 保存专业
	 * @param specialty
	 * @return
	 */
	public void save(Specialty specialty){
		checkCode(specialty.getOrgId(), specialty.getCode());
		specialty.setCreateTime(new Date());
		specialtyRepo.save(specialty);
	}
	
	//验证专业代码是否存在
	private void checkCode(Long orgId,String code){
		Specialty specialty = specialtyRepo.findByOrgIdAndCode(orgId, code);
		if(specialty != null){
			throw new RuntimeException("专业代码已存在");
		}
	}
	
	/**
	 * 更新专业
	 * @param specialty
	 */
	public void updateSpecialty(Specialty specialty){
		Specialty oldSpecialty = specialtyRepo.findOne(specialty.getId());
		if(!oldSpecialty.getCode().equals(specialty.getCode())){
			checkCode(specialty.getOrgId(), specialty.getCode());
		}
		specialty.setUpdateTime(new Date());
		specialtyRepo.save(specialty);
	}
	
	/**
	 * 根据Id删除专业
	 * @param ids
	 */
	public void deleteSpecialty(String ids){
		List<Long> specialtyIds = Stream.of(ids.split(",")).map(s->Long.parseLong(s.trim()))
                .collect(Collectors.toList());
		for(Long specialtyId: specialtyIds){
			specialtyRepo.delete(specialtyId);
		}
	}
}
